@@ -49,6 +49,8 @@
 #include "kernel.h"
 
 
+static void free_file_list(FileList* l);
+
 static void find_conflicting_xfree86_libraries(Options *,
                                                const char *,
                                                FileList *);
@@ -178,6 +180,7 @@ CommandList *build_command_list(Options *op, Package *p)
     /*
      * find any existing files that clash with what we're going to
      * install
+     * DON'T include files of type FILE_TYPE_NEWSYM
      */
     
     find_existing_files(p, l, installable_files | FILE_TYPE_SYMLINK);
@@ -193,6 +196,8 @@ CommandList *build_command_list(Options *op, Package *p)
             if (find_installed_file(op, l->filename[i])) {
                 ui_error(op, "The file '%s' already exists as part of this "
                          "driver installation.", l->filename[i]);
+                free_file_list(l);
+                free_command_list(op, c);
                 return NULL;
             }
         }
@@ -271,7 +276,21 @@ CommandList *build_command_list(Options *op, Package *p)
     /* create any needed symbolic links */
     
     for (i = 0; i < p->num_entries; i++) {
-        if (p->entries[i].flags & FILE_TYPE_SYMLINK) {
+        if ((p->entries[i].flags & FILE_TYPE_SYMLINK) ||
+            (p->entries[i].flags & FILE_TYPE_NEWSYM)) {
+            /* if it's a NEWSYM and the file already exists, don't add a command
+             * for it */
+            if (p->entries[i].flags & FILE_TYPE_NEWSYM) {
+                struct stat buf;
+                if(!stat(p->entries[i].dst, &buf) || errno != ENOENT) {
+                    ui_expert(op, "Not creating a symlink from %s to %s "
+                                  "because a file already exists at that path "
+                                  "or the path is inaccessible.",
+                                  p->entries[i].dst, p->entries[i].target);
+                    continue;
+                }
+            }
+
             add_command(c, SYMLINK_CMD, p->entries[i].dst,
                         p->entries[i].target);
         }
@@ -335,10 +354,7 @@ CommandList *build_command_list(Options *op, Package *p)
     }
 
     /* free the FileList */
-
-    for (i = 0; i < l->num; i++) free(l->filename[i]);
-    free(l->filename);
-    free(l);
+    free_file_list(l);
 
     return c;
 
@@ -346,6 +362,27 @@ CommandList *build_command_list(Options *op, Package *p)
 
 
 
+/*
+ * free_file_list() - free the file list
+ */
+
+static void free_file_list(FileList* l)
+{
+    int i;
+
+    if (!l) return;
+
+    for (i = 0; i < l->num; i++) {
+        nvfree(l->filename[i]);
+    }
+
+    nvfree((char *) l->filename);
+    nvfree((char *) l);
+    
+} /* free_file_list() */
+
+
+
 /*
  * free_command_list() - free the specified commandlist
  */
@@ -616,7 +653,6 @@ static void find_conflicting_kernel_modules(Options *op,
     char *paths[3];
     char *tmp = get_kernel_name(op);
 
-    memset(files, 0, sizeof(files));
     files[1].name = NULL;
     files[1].len = 0;
     paths[0] = op->kernel_module_installation_path;
